@@ -2,6 +2,7 @@ package oviewer
 
 import (
 	"context"
+	"os"
 	"path/filepath"
 	"reflect"
 	"testing"
@@ -10,7 +11,6 @@ import (
 )
 
 func TestRoot_toggle(t *testing.T) {
-	t.Parallel()
 	root := rootHelper(t)
 	ctx := context.Background()
 	var v bool
@@ -162,8 +162,8 @@ func Test_rangeBA(t *testing.T) {
 func Test_position(t *testing.T) {
 	t.Parallel()
 	type args struct {
-		height int
 		str    string
+		height int
 	}
 	tests := []struct {
 		name string
@@ -173,32 +173,32 @@ func Test_position(t *testing.T) {
 		{
 			name: "test1",
 			args: args{
-				height: 30,
 				str:    "1",
+				height: 30,
 			},
 			want: 1,
 		},
 		{
 			name: "test.5",
 			args: args{
-				height: 30,
 				str:    ".5",
+				height: 30,
 			},
 			want: 15,
 		},
 		{
 			name: "test20%",
 			args: args{
-				height: 30,
 				str:    "20%",
+				height: 30,
 			},
 			want: 6,
 		},
 		{
 			name: "test.3",
 			args: args{
-				height: 45,
 				str:    "30%",
+				height: 45,
 			},
 			want: 13.5,
 		},
@@ -207,7 +207,7 @@ func Test_position(t *testing.T) {
 		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			t.Parallel()
-			if got := calculatePosition(tt.args.height, tt.args.str); got != tt.want {
+			if got := calculatePosition(tt.args.str, tt.args.height); got != tt.want {
 				t.Errorf("position() = %v, want %v", got, tt.want)
 			}
 		})
@@ -215,7 +215,6 @@ func Test_position(t *testing.T) {
 }
 
 func TestRoot_setJumpTarget(t *testing.T) {
-	t.Parallel()
 	root := rootHelper(t)
 	type fields struct{}
 	type args struct {
@@ -228,6 +227,22 @@ func TestRoot_setJumpTarget(t *testing.T) {
 		wantBool bool
 		want     int
 	}{
+		{
+			name: "testJumpTargetBlank",
+			args: args{
+				input: "",
+			},
+			wantBool: false,
+			want:     0,
+		},
+		{
+			name: "testJumpTargetBlank2",
+			args: args{
+				input: "",
+			},
+			wantBool: false,
+			want:     0,
+		},
 		{
 			name: "testJumpTargetOutOfRange",
 			args: args{
@@ -439,10 +454,6 @@ func TestRoot_toggleColumnMode(t *testing.T) {
 }
 
 func TestRoot_toggleMouse(t *testing.T) {
-	tcellNewScreen = fakeScreen
-	defer func() {
-		tcellNewScreen = tcell.NewScreen
-	}()
 	type fields struct {
 		disableMouse bool
 	}
@@ -595,6 +606,13 @@ func TestRoot_setSkipLines(t *testing.T) {
 		args args
 		want int
 	}{
+		{
+			name: "testSetSkipErr",
+			args: args{
+				input: "err",
+			},
+			want: 0,
+		},
 		{
 			name: "testSetSkipLines1",
 			args: args{
@@ -631,6 +649,13 @@ func TestRoot_setSectioNum(t *testing.T) {
 		args args
 		want int
 	}{
+		{
+			name: "testSetSectionNumNil",
+			args: args{
+				input: "",
+			},
+			want: 0,
+		},
 		{
 			name: "testSetSectionNum1",
 			args: args{
@@ -877,7 +902,6 @@ func TestRoot_ColumnDelimiterWrapMode(t *testing.T) {
 	defer func() {
 		tcellNewScreen = tcell.NewScreen
 	}()
-	root := rootFileReadHelper(t, filepath.Join(testdata, "MOCK_DATA.csv"))
 	type fields struct {
 		wrapMode     bool
 		x            int
@@ -922,7 +946,10 @@ func TestRoot_ColumnDelimiterWrapMode(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
+			root := rootFileReadHelper(t, filepath.Join(testdata, "MOCK_DATA.csv"))
+			root.prepareScreen()
 			ctx := context.Background()
+			root.prepareDraw(ctx)
 			root.setDelimiter(",")
 			root.Doc.ColumnMode = true
 			root.Doc.WrapMode = tt.fields.wrapMode
@@ -1011,6 +1038,7 @@ func TestRoot_ColumnWidthWrapMode(t *testing.T) {
 			root.Doc.WrapMode = tt.fields.wrapMode
 			root.Doc.x = tt.fields.x
 			root.Doc.columnCursor = tt.fields.columnCursor
+			root.Doc.setColumnWidths()
 			root.prepareDraw(ctx)
 			root.toggleWrapMode(ctx)
 			if root.Doc.WrapMode != tt.want.wrapMode {
@@ -1049,6 +1077,7 @@ func TestRoot_Mark(t *testing.T) {
 		if !reflect.DeepEqual(root.Doc.marked, []int{10}) {
 			t.Errorf("removeAllMark() = %#v, want %#v", root.Doc.marked, []int{10})
 		}
+		root.removeMark(ctx)
 		root.Doc.topLN = 2
 		root.addMark(ctx)
 		if !reflect.DeepEqual(root.Doc.marked, []int{10, 2}) {
@@ -1068,7 +1097,6 @@ func TestRoot_markNext(t *testing.T) {
 	}()
 	root := rootFileReadHelper(t, filepath.Join(testdata, "test3.txt"))
 	root.prepareScreen()
-	root.Doc.marked = []int{1, 3, 5}
 	tests := []struct {
 		name        string
 		markedPoint int
@@ -1093,6 +1121,8 @@ func TestRoot_markNext(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
+			root.markNext(context.Background()) // no marked
+			root.Doc.marked = []int{1, 3, 5}
 			root.Doc.markedPoint = tt.markedPoint
 			root.markNext(context.Background())
 			if root.Doc.topLN != tt.wantLine {
@@ -1109,7 +1139,6 @@ func TestRoot_markPrev(t *testing.T) {
 	}()
 	root := rootFileReadHelper(t, filepath.Join(testdata, "test3.txt"))
 	root.prepareScreen()
-	root.Doc.marked = []int{1, 3, 5}
 	tests := []struct {
 		name        string
 		markedPoint int
@@ -1134,6 +1163,8 @@ func TestRoot_markPrev(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
+			root.markPrev(context.Background()) // no marked
+			root.Doc.marked = []int{1, 3, 5}
 			root.Doc.markedPoint = tt.markedPoint
 			root.markPrev(context.Background())
 			if root.Doc.topLN != tt.wantLine {
@@ -1192,3 +1223,285 @@ func TestRoot_setWriteBA(t *testing.T) {
 		})
 	}
 }
+
+func TestRoot_modeConfig(t *testing.T) {
+	type args struct {
+		modeName string
+	}
+	tests := []struct {
+		name    string
+		args    args
+		want    general
+		wantErr bool
+	}{
+		{
+			name: "testModeConfig error",
+			args: args{
+				modeName: "test",
+			},
+			want:    general{},
+			wantErr: true,
+		},
+		{
+			name: "testModeConfig general",
+			args: args{
+				modeName: "general",
+			},
+			want:    general{},
+			wantErr: false,
+		},
+		{
+			name: "testModeConfig a",
+			args: args{
+				modeName: "a",
+			},
+			want:    general{},
+			wantErr: false,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			root := rootHelper(t)
+			root.Config = Config{
+				General: general{},
+				Mode: map[string]general{
+					"a": {},
+					"b": {},
+				},
+			}
+			got, err := root.modeConfig(tt.args.modeName)
+			if (err != nil) != tt.wantErr {
+				t.Errorf("Root.modeConfig() error = %v, wantErr %v", err, tt.wantErr)
+				return
+			}
+			if !reflect.DeepEqual(got, tt.want) {
+				t.Errorf("Root.modeConfig() = %v, want %v", got, tt.want)
+			}
+		})
+	}
+}
+
+func TestRoot_suspend(t *testing.T) {
+	root := rootHelper(t)
+	shell := os.Getenv("SHELL")
+	if got := root.shellSuspendResume(shell); got != nil {
+		t.Errorf("shellSuspendResume() = %v, want %v", got, nil)
+	}
+	if got := root.shellSuspendResume(""); got != nil {
+		t.Errorf("shellSuspendResume() = %v, want %v", got, nil)
+	}
+}
+
+func TestRoot_follow(t *testing.T) {
+	tcellNewScreen = fakeScreen
+	defer func() {
+		tcellNewScreen = tcell.NewScreen
+	}()
+	type fields struct {
+		wrapMode bool
+	}
+	type args struct {
+		fileNames []string
+	}
+	tests := []struct {
+		name   string
+		fields fields
+		args   args
+		want   int
+	}{
+		{
+			name: "testFollow1",
+			fields: fields{
+				wrapMode: false,
+			},
+			args: args{
+				fileNames: []string{filepath.Join(testdata, "test3.txt")},
+			},
+			want: 12322,
+		},
+		{
+			name: "testFollow2",
+			fields: fields{
+				wrapMode: true,
+			},
+			args: args{
+				fileNames: []string{filepath.Join(testdata, "test3.txt")},
+			},
+			want: 12322,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			root := rootFileReadHelper(t, tt.args.fileNames...)
+			root.Doc.WrapMode = tt.fields.wrapMode
+			ctx := context.Background()
+			root.prepareScreen()
+			root.everyUpdate(ctx)
+			root.follow(ctx)
+			if root.Doc.topLN != tt.want {
+				t.Errorf("follow() topLN = %v, want %v", root.Doc.topLN, tt.want)
+			}
+			root.Cancel(ctx)
+		})
+	}
+}
+
+func TestRoot_followAll(t *testing.T) {
+	tcellNewScreen = fakeScreen
+	defer func() {
+		tcellNewScreen = tcell.NewScreen
+	}()
+	type fields struct {
+		wrapMode bool
+	}
+	type args struct {
+		fileNames []string
+	}
+	tests := []struct {
+		name   string
+		fields fields
+		args   args
+		want   int
+	}{
+		{
+			name: "testFollowAll1",
+			fields: fields{
+				wrapMode: false,
+			},
+			args: args{
+				fileNames: []string{
+					filepath.Join(testdata, "test3.txt"),
+					filepath.Join(testdata, "test.txt"),
+				},
+			},
+			want: 12322,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			root := rootFileReadHelper(t, tt.args.fileNames...)
+			root.General.FollowAll = true
+			root.Doc.WrapMode = tt.fields.wrapMode
+			ctx := context.Background()
+			root.prepareScreen()
+			root.everyUpdate(ctx)
+			root.follow(ctx)
+			if root.Doc.topLN != tt.want {
+				t.Errorf("follow() topLN = %v, want %v", root.Doc.topLN, tt.want)
+			}
+		})
+	}
+}
+
+func TestRoot_WriteQuit(t *testing.T) {
+	tcellNewScreen = fakeScreen
+	defer func() {
+		tcellNewScreen = tcell.NewScreen
+	}()
+	type fields struct {
+		fileNames        []string
+		sectionDelimiter string
+		HideOtherSection bool
+	}
+	tests := []struct {
+		name   string
+		fields fields
+		want   int
+	}{
+		{
+			name: "testWriteQuit1",
+			fields: fields{
+				fileNames:        []string{filepath.Join(testdata, "section.txt")},
+				sectionDelimiter: "",
+				HideOtherSection: false,
+			},
+			want: 0,
+		},
+		{
+			name: "testWriteQuit2",
+			fields: fields{
+				fileNames:        []string{filepath.Join(testdata, "section.txt")},
+				sectionDelimiter: "",
+				HideOtherSection: true,
+			},
+			want: 0,
+		},
+		{
+			name: "testWriteQuit3",
+			fields: fields{
+				fileNames:        []string{filepath.Join(testdata, "section.txt")},
+				sectionDelimiter: "^#",
+				HideOtherSection: true,
+			},
+			want: 2,
+		},
+		{
+			name: "testWriteQuit4",
+			fields: fields{
+				fileNames:        []string{filepath.Join(testdata, "section.txt")},
+				sectionDelimiter: "^notfound",
+				HideOtherSection: true,
+			},
+			want: 0,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			root := rootFileReadHelper(t, tt.fields.fileNames...)
+			root.prepareScreen()
+			ctx := context.Background()
+			root.setSectionDelimiter(tt.fields.sectionDelimiter)
+			root.prepareDraw(context.Background())
+			root.Doc.HideOtherSection = tt.fields.HideOtherSection
+			root.WriteQuit(ctx)
+			if got := root.AfterWriteOriginal; got != tt.want {
+				t.Errorf("WriteQuit() AfterWriteOriginal = %v, want %v", got, tt.want)
+			}
+		})
+	}
+}
+
+func TestRoot_tailSection(t *testing.T) {
+	tcellNewScreen = fakeScreen
+	defer func() {
+		tcellNewScreen = tcell.NewScreen
+	}()
+	type fields struct {
+		fileNames        []string
+		sectionDelimiter string
+	}
+	tests := []struct {
+		name   string
+		fields fields
+		want   int
+	}{
+		{
+			name: "testTailSection",
+			fields: fields{
+				fileNames:        []string{filepath.Join(testdata, "section.txt")},
+				sectionDelimiter: "^#",
+			},
+			want: 8,
+		},
+		{
+			name: "testTailSectionErr",
+			fields: fields{
+				fileNames:        []string{filepath.Join(testdata, "section.txt")},
+				sectionDelimiter: "^testtesttest",
+			},
+			want: 0,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			root := rootFileReadHelper(t, tt.fields.fileNames...)
+			root.prepareScreen()
+			ctx := context.Background()
+			root.setSectionDelimiter(tt.fields.sectionDelimiter)
+			root.tailSection(ctx)
+			if got := root.Doc.lastSectionPosNum; got != tt.want {
+				t.Errorf("tailSection() lastSectionPosNum = %v, want %v", got, tt.want)
+			}
+		})
+	}
+}
