@@ -521,5 +521,9 @@ func (m *Document) setColumnWidths() {
 	for n, line := range lines {
 		buf[n] = string(line)
 	}
+	// Stop guessing if valid row count is not reached.
+	if !m.BufEOF() && len(buf) < 20 {
+		return
+	}
 	m.columnWidths = guesswidth.Positions(buf, header, 2)
 }
