@@ -157,6 +157,13 @@ func (root *Root) everyUpdate(ctx context.Context) {
 	if atomic.SwapInt32(&root.Doc.watchRestart, 0) == 1 {
 		root.watchControl()
 	}
+
+	if root.quitSmallCountDown > 0 {
+		root.quitSmallCountDown--
+		if root.DocumentLen() == 2 && root.Doc.documentType == DocFilter && root.docSmall() {
+			root.WriteQuit(ctx)
+		}
+	}
 }
 
 // keyEvent processes key events.
