@@ -5,6 +5,7 @@ import (
 	"fmt"
 	"io"
 	"log"
+	"strings"
 )
 
 // filterDocument is a document for filtering.
@@ -25,19 +26,11 @@ func (root *Root) Filter(str string, nonMatch bool) {
 }
 
 // filter filters the document by the input value.
-func (root *Root) filter(ctx context.Context) {
-	searcher := root.setSearcher(root.input.value, root.Config.CaseSensitive)
+func (root *Root) filter(ctx context.Context, str string) {
+	searcher := root.setSearcher(str, root.Config.CaseSensitive)
 	if searcher == nil {
-		if root.Doc.jumpTargetSection {
-			root.Doc.jumpTargetHeight = 0
-		}
 		return
 	}
-	word := root.searcher.String()
-	if root.Doc.nonMatch {
-		word = fmt.Sprintf("!%s", word)
-	}
-	root.setMessagef("filter:%v", word)
 	root.filterDocument(ctx, searcher)
 }
 
@@ -50,33 +43,36 @@ func (root *Root) filterDocument(ctx context.Context, searcher Searcher) {
 		return
 	}
 	render.documentType = DocFilter
-	render.FileName = fmt.Sprintf("filter:%s:%v", m.FileName, searcher.String())
-	render.Caption = fmt.Sprintf("%s:%v", m.FileName, searcher.String())
-	root.addDocument(ctx, render)
-	render.general = mergeGeneral(m.general, render.general)
+	match := searcher.String()
+	if m.nonMatch {
+		match = "!" + match
+	}
+	render.Caption = fmt.Sprintf("filter:%s", match)
+	msg := fmt.Sprintf("search:%s", match)
+	root.insertDocument(ctx, root.CurrentDoc, render)
+	render.general = mergeGeneral(root.Config.General, render.general)
+	render.regexpCompile()
 
 	render.nonMatch = m.nonMatch
 	render.Header = m.Header
 	render.SkipLines = m.SkipLines
-
 	filterDoc := &filterDocument{
 		Document: render,
 		w:        w,
 	}
 
 	// Copy the header
-	if render.Header > 0 {
-		for ln := render.SkipLines; ln < render.Header; ln++ {
-			line, err := m.Line(ln)
-			if err != nil {
-				break
-			}
-			render.lineNumMap.Store(ln, ln)
-			writeLine(w, line)
+	for ln := 0; ln < render.firstLine(); ln++ {
+		line, err := m.Line(ln)
+		if err != nil {
+			log.Println(err)
+			break
 		}
+		render.lineNumMap.Store(ln, ln)
+		writeLine(w, line)
 	}
 	go m.filterWriter(ctx, searcher, m.firstLine(), filterDoc)
-	root.setMessagef("filter:%v", searcher.String())
+	root.setMessagef(msg)
 }
 
 // filterWriter searches and writes to filterDoc.
@@ -97,6 +93,7 @@ func (m *Document) filterWriter(ctx context.Context, searcher Searcher, startLN
 		line, err := m.Line(lineNum)
 		if err != nil {
 			// deleted?
+			log.Println(err)
 			break
 		}
 		filterDoc.lineNumMap.Store(renderLN, lineNum)
@@ -109,7 +106,20 @@ func (m *Document) filterWriter(ctx context.Context, searcher Searcher, startLN
 
 // closeAllFilter closes all filter documents.
 func (root *Root) closeAllFilter(ctx context.Context) {
-	root.closeAllDocument(ctx, DocFilter)
+	if root.DocumentLen() == 1 {
+		root.setMessage("only this document")
+		return
+	}
+
+	docNum, closed := root.closeAllDocumentsOfType(DocFilter)
+	root.setDocumentNum(ctx, docNum)
+
+	if len(closed) == 0 {
+		root.setMessage("no document to close")
+		return
+	}
+	root.setMessageLogf("close %s", strings.Join(closed, ", "))
+
 }
 
 // writeLine writes a line to w.
