@@ -185,52 +185,63 @@ func (input *Input) reset() {
 	input.cursorX = 0
 }
 
-// inputCaseSensitive toggles case sensitivity.
-func (root *Root) inputCaseSensitive() {
-	root.Config.CaseSensitive = !root.Config.CaseSensitive
-	if root.Config.CaseSensitive {
-		root.Config.SmartCaseSensitive = false
-	}
-}
-
-// inputSmartCaseSensitive toggles case sensitivity.
-func (root *Root) inputSmartCaseSensitive() {
-	root.Config.SmartCaseSensitive = !root.Config.SmartCaseSensitive
-	if root.Config.SmartCaseSensitive {
-		root.Config.CaseSensitive = false
-	}
-}
-
-// inputIncSearch toggles incremental search.
-func (root *Root) inputIncSearch() {
-	root.Config.Incsearch = !root.Config.Incsearch
-}
-
-// inputRegexpSearch toggles regexp search.
-func (root *Root) inputRegexpSearch() {
-	root.Config.RegexpSearch = !root.Config.RegexpSearch
-}
-
-func (root *Root) inputNonMatch() {
-	root.Doc.nonMatch = !root.Doc.nonMatch
-}
-
-// inputPrevious searches the previous history.
-func (root *Root) inputPrevious() {
-	input := root.input
+func (input *Input) previous() {
 	input.value = input.Event.Up(input.value)
 	runes := []rune(input.value)
 	input.cursorX = stringWidth(string(runes))
 }
 
-// inputNext searches the next history.
-func (root *Root) inputNext() {
-	input := root.input
+func (input *Input) next() {
 	input.value = input.Event.Down(input.value)
 	runes := []rune(input.value)
 	input.cursorX = stringWidth(string(runes))
 }
 
+// inputCaseSensitive toggles case sensitivity.
+func (root *Root) inputCaseSensitive(context.Context) {
+	root.Config.CaseSensitive = !root.Config.CaseSensitive
+	if root.Config.CaseSensitive {
+		root.Config.SmartCaseSensitive = false
+	}
+	root.setPromptOpt()
+}
+
+// inputSmartCaseSensitive toggles case sensitivity.
+func (root *Root) inputSmartCaseSensitive(context.Context) {
+	root.Config.SmartCaseSensitive = !root.Config.SmartCaseSensitive
+	if root.Config.SmartCaseSensitive {
+		root.Config.CaseSensitive = false
+	}
+	root.setPromptOpt()
+}
+
+// inputIncSearch toggles incremental search.
+func (root *Root) inputIncSearch(context.Context) {
+	root.Config.Incsearch = !root.Config.Incsearch
+	root.setPromptOpt()
+}
+
+// inputRegexpSearch toggles regexp search.
+func (root *Root) inputRegexpSearch(context.Context) {
+	root.Config.RegexpSearch = !root.Config.RegexpSearch
+	root.setPromptOpt()
+}
+
+func (root *Root) inputNonMatch(context.Context) {
+	root.Doc.nonMatch = !root.Doc.nonMatch
+	root.setPromptOpt()
+}
+
+// inputPrevious searches the previous history.
+func (root *Root) inputPrevious(context.Context) {
+	root.input.previous()
+}
+
+// inputNext searches the next history.
+func (root *Root) inputNext(context.Context) {
+	root.input.next()
+}
+
 // stringWidth returns the number of widths of the input.
 // Tab is 2 characters.
 func stringWidth(str string) int {
