@@ -4,10 +4,9 @@ import (
 	"context"
 	"log"
 	"sync/atomic"
-	"time"
 )
 
-// lastLineMargin is the margin of the last line
+// lastLineMargin is the margin of the last line.
 const lastLineMargin = 1
 
 // moveLine moves to the specified line.
@@ -15,6 +14,7 @@ func (m *Document) moveLine(lN int) int {
 	if m.BufEndNum() == 0 {
 		return 0
 	}
+	m.showGotoF = true
 
 	if endLN := m.BufEndNum() - 1; lN > endLN {
 		lN = endLN
@@ -57,7 +57,6 @@ func (m *Document) moveLineNth(lN int, nTh int) (int, int) {
 
 	m.topLN = lN
 	m.topLX = listX[nTh]
-
 	return lN, nTh
 }
 
@@ -129,7 +128,7 @@ func numOfReverseSlice(listX []int, x int) int {
 // bottomLineNum returns the display start line
 // when the last line number as an argument.
 func (m *Document) bottomLineNum(lN int, height int) (int, int) {
-	if lN < m.headerLen {
+	if lN < m.headerHeight {
 		return 0, 0
 	}
 	if !m.WrapMode {
@@ -248,79 +247,56 @@ func leftX(width int, lc contents) []int {
 	return listX
 }
 
-// moveNextSection moves to the next section.
-func (m *Document) moveNextSection() error {
-	// Move by page, if there is no section delimiter.
-	if m.SectionDelimiter == "" {
-		m.movePgDn()
-		return nil
-	}
-
-	lN, err := m.nextSection(m.topLN + m.firstLine())
-	if err != nil {
-		m.movePgDn()
-		return ErrNoDelimiter
-	}
-	m.moveLine((lN - m.firstLine() + m.SectionHeaderNum) + m.SectionStartPosition)
-	return nil
-}
-
-// nextSection returns the line number of the previous section.
-func (m *Document) nextSection(n int) (int, error) {
-	ctx := context.Background()
-	defer ctx.Done()
-
-	lN := n + (1 - m.SectionStartPosition)
+// nextSection returns the line number of the next section.
+func (m *Document) nextSection(ctx context.Context, lN int) (int, error) {
 	searcher := NewSearcher(m.SectionDelimiter, m.SectionDelimiterReg, true, true)
-	return m.SearchLine(ctx, searcher, lN)
-}
-
-// movePrevSection moves to the previous section.
-func (m *Document) movePrevSection() error {
-	// Move by page, if there is no section delimiter.
-	if m.SectionDelimiter == "" {
-		m.movePgUp()
-		return nil
-	}
-
-	lN, err := m.prevSection(m.topLN + m.firstLine() - m.SectionHeaderNum)
-	if err != nil {
-		m.moveTop()
-		return err
-	}
-	lN = (lN - m.firstLine()) + m.SectionStartPosition
-	lN = max(lN, m.BufStartNum())
-	m.moveLine(lN + m.SectionHeaderNum)
-	return nil
+	return m.SearchLine(ctx, searcher, lN+1)
 }
 
 // prevSection returns the line number of the previous section.
-func (m *Document) prevSection(n int) (int, error) {
-	// TODO: Timeout should be specified in the caller instead of here.
-	// If it takes a long time to find the section header,
-	// it will freeze during that time, so a timeout is set.
-	ctx := context.Background()
-	ctx, cancel := context.WithTimeout(ctx, sectionTimeOut*time.Millisecond)
-	defer cancel()
-
+func (m *Document) prevSection(ctx context.Context, lN int) (int, error) {
 	searcher := NewSearcher(m.SectionDelimiter, m.SectionDelimiterReg, true, true)
-	lN := n - (1 + m.SectionStartPosition)
-	return m.BackSearchLine(ctx, searcher, lN)
+	return m.BackSearchLine(ctx, searcher, lN-1)
+}
+
+// moveNextSection moves to the next section.
+func (m *Document) moveNextSection(ctx context.Context) error {
+	if m.SectionDelimiter == "" {
+		return ErrNoDelimiter
+	}
+
+	lN, err := m.nextSection(ctx, m.topLN+m.firstLine()-m.SectionStartPosition)
+	if err != nil {
+		return ErrNoMoreSection
+	}
+	m.moveLine((lN - m.firstLine() + m.SectionStartPosition))
+	return nil
+}
+
+// movePrevSection moves to the previous section.
+func (m *Document) movePrevSection(ctx context.Context) error {
+	return m.movePrevSectionLN(ctx, m.topLN)
+}
+
+// movePrevSectionLN moves to the previous section by line number.
+func (m *Document) movePrevSectionLN(ctx context.Context, start int) error {
+	if m.SectionDelimiter == "" {
+		return ErrNoDelimiter
+	}
+
+	lN, err := m.prevSection(ctx, start+m.firstLine()-m.SectionStartPosition)
+	if err != nil {
+		return ErrNoMoreSection
+	}
+	lN = (lN - m.firstLine() + m.SectionStartPosition)
+	lN = max(lN, m.BufStartNum())
+	m.moveLine(lN)
+	return nil
 }
 
 // moveLastSection moves to the last section.
-func (m *Document) moveLastSection() {
-	ctx := context.Background()
-	defer ctx.Done()
-
-	// +1 to avoid if the bottom line is a session delimiter.
-	lN := m.BufEndNum() - 2
-	searcher := NewSearcher(m.SectionDelimiter, m.SectionDelimiterReg, true, true)
-	n, err := m.BackSearchLine(ctx, searcher, lN)
-	if err != nil {
+func (m *Document) moveLastSection(ctx context.Context) {
+	if err := m.movePrevSectionLN(ctx, m.BufEndNum()+1); err != nil {
 		log.Printf("last section:%v", err)
-		return
 	}
-	n = (n - m.firstLine()) + m.SectionStartPosition
-	m.moveLine(n)
 }
