@@ -67,13 +67,14 @@ type Root struct {
 	// minStartX is the minimum start position of x.
 	minStartX int
 
+	// quitSmallCountDown is the countdown to quit if the output fits on one screen.
+	quitSmallCountDown int
+
 	// mu controls the RWMutex.
 	mu sync.RWMutex
 
 	// skipDraw is set to true when the mouse cursor just moves (no event occurs).
 	skipDraw bool
-	// Show document number
-	showDocNum bool
 }
 
 // SCR contains the screen information.
@@ -261,6 +262,8 @@ type Config struct {
 	IsWriteOriginal bool
 	// QuitSmall Quit if the output fits on one screen.
 	QuitSmall bool
+	// QuitSmallFilter Quit if the output fits on one screen and the filter is applied.
+	QuitSmallFilter bool
 	// CaseSensitive is case-sensitive if true.
 	CaseSensitive bool
 	// SmartCaseSensitive is lowercase search ignores case, if true.
@@ -345,6 +348,10 @@ var (
 // Does not track mouse movements except when dragging.
 const MouseFlags = tcell.MouseDragEvents
 
+// QuitSmallCountDown is the countdown to quit if the output fits on one screen.
+// UpdateInterval(50 * time.Millisecond) * 10 = 500ms.
+const QuitSmallCountDown = 10
+
 // MaxWriteLog is the maximum number of lines to output to the log
 // when the debug flag is enabled.
 const MaxWriteLog int = 10
@@ -372,6 +379,10 @@ var (
 	ErrSignalCatch = errors.New("signal catch")
 	// ErrAlreadyClose indicates that it is already closed.
 	ErrAlreadyClose = errors.New("already closed")
+	// ErrCannotClose indicates that it cannot be closed.
+	ErrCannotClose = errors.New("cannot close")
+	// ErrRequestClose indicates that the request is to close.
+	ErrRequestClose = errors.New("request close")
 	// ErrNoColumn indicates that cursor specified a nonexistent column.
 	ErrNoColumn = errors.New("no column")
 	// ErrNoDelimiter indicates that the line containing the delimiter could not be found.
@@ -430,10 +441,6 @@ func NewOviewer(docs ...*Document) (*Root, error) {
 	}
 	root.logDoc = logDoc
 
-	if root.DocumentLen() > 1 {
-		root.showDocNum = true
-	}
-
 	return root, nil
 }
 
@@ -610,8 +617,8 @@ func (root *Root) Run() error {
 	ctx := context.Background()
 	ctx, cancel := context.WithCancel(ctx)
 	defer cancel()
-
 	defer root.Close()
+
 	watcher, err := fsnotify.NewWatcher()
 	if err != nil {
 		return fmt.Errorf("failed to create watcher: %w", err)
@@ -619,58 +626,19 @@ func (root *Root) Run() error {
 	defer watcher.Close()
 	root.SetWatcher(watcher)
 
-	// Do not set the key bindings in NewOviewer
-	// because it is done after loading the config.
-	keyBind, err := root.setKeyConfig(ctx)
-	if err != nil {
+	if err := root.prepareRun(ctx); err != nil {
 		return err
 	}
-	help, err := NewHelp(keyBind)
-	if err != nil {
-		return err
-	}
-	root.helpDoc = help
 
-	if !root.Config.DisableMouse {
-		root.Screen.EnableMouse(MouseFlags)
-	}
-
-	root.optimizedMan()
-	if root.General.Caption != "" {
-		root.Doc.Caption = root.General.Caption
-	}
-
-	root.setModeConfig()
-	for n, doc := range root.DocList {
-		doc.general = root.Config.General
-		doc.regexpCompile()
-
-		if doc.FollowName {
-			doc.FollowMode = true
-		}
-		if doc.ColumnWidth {
-			doc.ColumnMode = true
-		}
-		w := ""
-		if doc.general.WatchInterval > 0 {
-			doc.watchMode()
-			w = "(watch)"
-		}
-		log.Printf("open [%d]%s%s", n, doc.FileName, w)
-	}
-
-	root.ViewSync(ctx)
-	// Exit if fits on screen
-	if root.QuitSmall && root.docSmall() {
+	// Quit if fits on screen.
+	if root.QuitSmall && root.DocumentLen() == 1 && root.docSmall() {
 		root.IsWriteOriginal = true
 		return nil
 	}
 
 	sigs := make(chan os.Signal, 1)
 	signal.Notify(sigs, os.Interrupt, syscall.SIGTERM, syscall.SIGHUP, syscall.SIGINT)
-
 	sigSuspend := registerSIGTSTP()
-
 	quitChan := make(chan struct{})
 
 	go func() {
@@ -693,8 +661,45 @@ func (root *Root) Run() error {
 	}
 }
 
-// optimizedMan optimizes execution with the Man command.
-func (root *Root) optimizedMan() {
+// prepareRun prepares to run the oviewer.
+func (root *Root) prepareRun(ctx context.Context) error {
+	// Do not set the key bindings in NewOviewer
+	// because it is done after loading the config.
+	keyBind, err := root.setKeyConfig(ctx)
+	if err != nil {
+		return err
+	}
+	help, err := NewHelp(keyBind)
+	if err != nil {
+		return err
+	}
+	root.helpDoc = help
+
+	if !root.Config.DisableMouse {
+		root.Screen.EnableMouse(MouseFlags)
+	}
+
+	root.setCaption()
+
+	root.setViewModeConfig()
+	root.prepareAllDocuments()
+	// Quit by filter result. This is evaluated lazily.
+	if root.QuitSmallFilter {
+		root.quitSmallCountDown = QuitSmallCountDown
+		root.QuitSmall = false
+	}
+	root.ViewSync(ctx)
+	return nil
+}
+
+// setCaption sets the caption.
+// optimizes execution with the Man command.
+func (root *Root) setCaption() {
+	if root.General.Caption != "" {
+		root.Doc.Caption = root.General.Caption
+		return
+	}
+
 	// Call from man command.
 	manPN := os.Getenv("MAN_PN")
 	if len(manPN) == 0 {
@@ -704,8 +709,8 @@ func (root *Root) optimizedMan() {
 	root.Doc.Caption = manPN
 }
 
-// setModeConfig sets mode config.
-func (root *Root) setModeConfig() {
+// setViewModeConfig sets view mode config.
+func (root *Root) setViewModeConfig() {
 	list := make([]string, 0, len(root.Config.Mode)+1)
 	list = append(list, "general")
 	for name := range root.Config.Mode {
@@ -714,6 +719,27 @@ func (root *Root) setModeConfig() {
 	root.input.ModeCandidate.list = list
 }
 
+// prepareAllDocuments prepares all documents.
+func (root *Root) prepareAllDocuments() {
+	for n, doc := range root.DocList {
+		doc.general = root.Config.General
+		doc.regexpCompile()
+
+		if doc.FollowName {
+			doc.FollowMode = true
+		}
+		if doc.ColumnWidth {
+			doc.ColumnMode = true
+		}
+		w := ""
+		if doc.general.WatchInterval > 0 {
+			doc.watchMode()
+			w = "(watch)"
+		}
+		log.Printf("open [%d]%s%s", n, doc.FileName, w)
+	}
+}
+
 // Close closes the oviewer.
 func (root *Root) Close() {
 	root.Screen.Fini()
@@ -925,9 +951,6 @@ func mergeGeneral(src general, dst general) general {
 
 // docSmall returns with bool whether the file to display fits on the screen.
 func (root *Root) docSmall() bool {
-	if len(root.DocList) > 1 {
-		return false
-	}
 	root.prepareScreen()
 	m := root.Doc
 	if !m.BufEOF() {
@@ -960,6 +983,9 @@ func (root *Root) writeOriginal(output io.Writer) {
 		m.bottomLN = m.BufEndNum()
 	}
 
+	ctx := context.Background()
+	root.prepareDraw(ctx)
+
 	// header
 	header := max(0, root.scr.headerLN)
 	headerEnd := max(0, root.scr.headerEnd)
@@ -989,10 +1015,14 @@ func (root *Root) writeOriginal(output io.Writer) {
 
 // WriteLog write to the log terminal.
 func (root *Root) WriteLog() {
+	root.writeLog(os.Stdout)
+}
+
+func (root *Root) writeLog(w io.Writer) {
 	m := root.logDoc
 	start := max(0, m.BufEndNum()-MaxWriteLog)
 	end := m.BufEndNum()
-	if err := m.Export(os.Stdout, start, end); err != nil {
+	if err := m.Export(w, start, end); err != nil {
 		log.Println(err)
 	}
 }
